"""AgentRecorder - Agent 轨迹录制工具

将任意 Agent 框架的执行日志转换为标准化轨迹格式。
"""

import logging

__version__ = "0.1.0"

from agentrecorder.recorder import Recorder
from agentrecorder.schema import Trajectory, Step, ToolCall, ToolResult

logging.getLogger(__name__).addHandler(logging.NullHandler())

__all__ = ["Recorder", "Trajectory", "Step", "ToolCall", "ToolResult", "__version__"]



"""Agent 框架适配器 — 注册表模式.

通过 register_adapter() 注册自定义适配器，通过 get_adapter() 获取。
"""

from agentrecorder.adapters.base import BaseAdapter
from agentrecorder.adapters.openhands import OpenHandsAdapter
from agentrecorder.adapters.sweagent import SWEAgentAdapter

_ADAPTER_REGISTRY: dict[str, type[BaseAdapter]] = {}


def register_adapter(name: str, adapter_cls: type[BaseAdapter]) -> None:
    """注册一个适配器.

    Args:
        name: 适配器名称 (如 "openhands", "swe-agent")
        adapter_cls: 适配器类（必须继承 BaseAdapter）
    """
    _ADAPTER_REGISTRY[name] = adapter_cls


def get_adapter(name: str) -> BaseAdapter | None:
    """按名称获取适配器实例.

    Args:
        name: 适配器名称

    Returns:
        适配器实例，未找到时返回 None
    """
    cls = _ADAPTER_REGISTRY.get(name)
    return cls() if cls else None


def list_adapters() -> list[str]:
    """列出所有已注册的适配器名称."""
    return list(_ADAPTER_REGISTRY.keys())


# 注册内置适配器
register_adapter("openhands", OpenHandsAdapter)
register_adapter("swe-agent", SWEAgentAdapter)
register_adapter("sweagent", SWEAgentAdapter)  # 别名

__all__ = [
    "BaseAdapter",
    "OpenHandsAdapter",
    "SWEAgentAdapter",
    "register_adapter",
    "get_adapter",
    "list_adapters",
]



"""SWE-agent 适配器.

将 SWE-agent 框架的执行日志转换为标准轨迹格式。

SWE-agent 日志格式说明:
- 日志通常为 JSON 文件，包含完整的 trajectory 数据
- 包含 history 数组，每个元素是一个 (action, observation) 对
- 元数据包含 model_name, instance_id, model_stats 等
- trajectory 目录结构: <instance_id>/<run_id>/trajectory.json
"""

import json
import logging
from datetime import datetime, timezone
from pathlib import Path

from agentrecorder.adapters.base import BaseAdapter
from agentrecorder.schema import Outcome, Step, ToolCall, Trajectory
from knowlyrcore import TaskInfo, ToolResult

logger = logging.getLogger(__name__)

# SWE-agent action 到标准工具名的映射
_ACTION_TOOL_MAP = {
    "bash": "bash",
    "edit": "edit_file",
    "open": "read_file",
    "scroll_up": "read_file",
    "scroll_down": "read_file",
    "search_dir": "search",
    "search_file": "search",
    "find_file": "search",
    "create": "write_file",
    "submit": "submit",
    "exit": "finish",
    "think": "think",
}


class SWEAgentAdapter(BaseAdapter):
    """SWE-agent 框架适配器.

    支持解析 SWE-agent 的 JSON 格式轨迹文件。

    Example:
        >>> adapter = SWEAgentAdapter()
        >>> if adapter.validate("trajectories/instance/trajectory.json"):
        ...     trajectory = adapter.parse("trajectories/instance/trajectory.json")
    """

    def parse(self, log_path: str) -> Trajectory:
        """将 SWE-agent 日志解析为标准轨迹格式.

        SWE-agent 日志结构:
        - 单个 JSON 文件包含完整轨迹
        - "history" 字段: [[action_dict, observation_str], ...]
        - "info" 字段: 包含 model_stats, exit_status 等
        - "trajectory" 字段: 详细的中间状态

        Args:
            log_path: SWE-agent 轨迹文件路径 (JSON 格式)。

        Returns:
            标准化轨迹对象。
        """
        path = Path(log_path)
        with open(path, "r", encoding="utf-8") as f:
            data = json.load(f)

        task_info = self._extract_task(data)
        model = self._extract_model(data)
        steps = self._parse_history(data.get("history", []))
        outcome = self._determine_outcome(data, steps)
        metadata = self._extract_metadata(data)

        return Trajectory(
            task=task_info,
            agent="swe-agent",
            model=model,
            steps=steps,
            outcome=outcome,
            metadata=metadata,
        )

    def validate(self, log_path: str) -> bool:
        """验证是否为 SWE-agent 日志格式.

        检查规则:
        1. 文件必须是 .json 格式
        2. JSON 应包含 SWE-agent 特征字段 (history, info)

        Args:
            log_path: 待验证的文件路径。

        Returns:
            如果是 SWE-agent 日志格式则返回 True。
        """
        path = Path(log_path)
        if not path.exists() or path.suffix != ".json":
            return False

        try:
            with open(path, "r", encoding="utf-8") as f:
                data = json.load(f)
                # SWE-agent 轨迹通常包含 history 和 info 字段
                return "history" in data and "info" in data
        except (json.JSONDecodeError, OSError):
            return False

    def _extract_task(self, data: dict) -> TaskInfo:
        """从数据中提取任务信息."""
        info = data.get("info", {})
        instance_id = info.get("instance_id", data.get("instance_id", ""))
        problem_statement = data.get("problem_statement", "")

        return TaskInfo(
            task_id=instance_id,
            description=problem_statement,
            repo=info.get("repo", ""),
            base_commit=info.get("base_commit", ""),
            metadata={"source": "swe-agent"},
        )

    def _extract_model(self, data: dict) -> str:
        """提取模型名称."""
        info = data.get("info", {})
        return info.get("model_name", data.get("model_name_or_path", ""))

    def _parse_history(self, history: list) -> list[Step]:
        """将 history 数组转换为 Step 列表.

        SWE-agent history 格式:
        - 每个元素是 [action_dict, observation_str]
        - action_dict 包含 action 名称和参数
        - observation_str 是工具输出的字符串
        """
        steps = []
        now = datetime.now(timezone.utc).isoformat()

        for step_id, entry in enumerate(history):
            if not isinstance(entry, (list, tuple)) or len(entry) < 2:
                continue

            action_data, observation = entry[0], entry[1]

            # action_data 可以是 dict 或 str
            if isinstance(action_data, dict):
                action_name = action_data.get("action", "unknown")
                args = {k: v for k, v in action_data.items() if k != "action"}
                thought = action_data.get("thought", "")
            else:
                # 纯字符串命令
                action_name = "bash"
                args = {"command": str(action_data)}
                thought = ""

            # 跳过 think 步骤（仅思考，无工具调用）
            if action_name == "think":
                continue

            tool_name = _ACTION_TOOL_MAP.get(action_name, action_name)
            obs_str = str(observation) if observation else ""

            # 判断 exit_code
            exit_code = 0
            if action_name in ("bash", "run") and obs_str.startswith("Error"):
                exit_code = 1

            step = Step(
                step_id=len(steps),
                thought=thought,
                tool_call=ToolCall(name=tool_name, parameters=args),
                tool_result=ToolResult(output=obs_str, exit_code=exit_code),
                timestamp=now,
            )
            steps.append(step)

        return steps

    def _determine_outcome(self, data: dict, steps: list[Step]) -> Outcome:
        """判断执行结果."""
        info = data.get("info", {})
        exit_status = info.get("exit_status", "")

        success = exit_status == "submitted"

        # 模型统计
        model_stats = info.get("model_stats", {})
        total_tokens = (
            model_stats.get("tokens_sent", 0)
            + model_stats.get("tokens_received", 0)
        )

        return Outcome(
            success=success,
            total_steps=len(steps),
            total_tokens=total_tokens,
        )

    def _extract_metadata(self, data: dict) -> dict:
        """提取额外元数据."""
        info = data.get("info", {})
        metadata = {}

        if info.get("instance_id"):
            metadata["instance_id"] = info["instance_id"]
        if info.get("model_stats"):
            metadata["model_stats"] = info["model_stats"]
        if info.get("exit_status"):
            metadata["exit_status"] = info["exit_status"]
        if data.get("environment"):
            metadata["environment"] = data["environment"]

        return metadata



"""AgentRecorder CLI - 命令行界面."""

import sys
from pathlib import Path
from typing import Optional

import click

from agentrecorder import __version__
from agentrecorder.schema import Trajectory


@click.group()
@click.version_option(version=__version__, prog_name="knowlyr-recorder")
def main():
    """AgentRecorder - Agent 轨迹录制工具

    将 Agent 框架的执行日志转换为标准化轨迹格式。
    """
    pass


@main.command()
@click.argument("log_path", type=click.Path(exists=True))
@click.option(
    "-f",
    "--framework",
    type=click.Choice(["openhands", "swe-agent"]),
    required=True,
    help="Agent 框架名称",
)
@click.option(
    "-o",
    "--output",
    type=click.Path(),
    help="输出 JSONL 文件路径",
)
def convert(log_path: str, framework: str, output: Optional[str]):
    """将 Agent 日志转换为标准化轨迹格式

    LOG_PATH: Agent 日志文件路径
    """
    adapter = _get_adapter(framework)
    if adapter is None:
        click.echo(f"不支持的框架: {framework}", err=True)
        sys.exit(1)

    if not adapter.validate(log_path):
        click.echo(f"日志格式不匹配 {framework}: {log_path}", err=True)
        sys.exit(1)

    click.echo("正在转换日志...")
    click.echo(f"  文件: {log_path}")
    click.echo(f"  框架: {framework}")

    try:
        from agentrecorder.recorder import Recorder

        recorder = Recorder(adapter)
        trajectory = recorder.convert(log_path)

        if output:
            trajectory.to_jsonl(output)
            click.echo(f"转换成功: {output}")
            click.echo(f"  步骤数: {len(trajectory.steps)}")
            click.echo(f"  结果: {'成功' if trajectory.outcome.success else '失败'}")
        else:
            click.echo(trajectory.model_dump_json(indent=2))
    except NotImplementedError as e:
        click.echo(f"适配器尚未实现: {e}", err=True)
        sys.exit(1)
    except Exception as e:
        click.echo(f"转换失败: {e}", err=True)
        sys.exit(1)


@main.command()
@click.argument("log_path", type=click.Path(exists=True))
@click.option(
    "-f",
    "--framework",
    type=click.Choice(["openhands", "swe-agent"]),
    help="指定框架进行验证（不指定则自动检测）",
)
def validate(log_path: str, framework: Optional[str]):
    """验证日志文件格式

    LOG_PATH: 待验证的日志文件路径
    """
    if framework:
        adapter = _get_adapter(framework)
        if adapter is None:
            click.echo(f"不支持的框架: {framework}", err=True)
            sys.exit(1)

        is_valid = adapter.validate(log_path)
        if is_valid:
            click.echo(f"有效: {log_path} 是 {framework} 格式")
        else:
            click.echo(f"无效: {log_path} 不是 {framework} 格式")
            sys.exit(1)
    else:
        click.echo(f"自动检测: {log_path}")
        matched = []
        for fw_name in ["openhands", "swe-agent"]:
            adapter = _get_adapter(fw_name)
            if adapter and adapter.validate(log_path):
                matched.append(fw_name)

        if matched:
            click.echo(f"  匹配框架: {', '.join(matched)}")
        else:
            click.echo("  未匹配任何已知框架格式")
            sys.exit(1)


@main.command()
def schema():
    """输出标准化轨迹的 JSON Schema"""
    click.echo(Trajectory.schema_json_example())


@main.command()
@click.argument("log_dir", type=click.Path(exists=True))
@click.option(
    "-f",
    "--framework",
    type=click.Choice(["openhands", "swe-agent"]),
    required=True,
    help="Agent 框架名称",
)
@click.option(
    "-o",
    "--output",
    type=click.Path(),
    required=True,
    help="输出 JSONL 文件路径",
)
@click.option(
    "-p",
    "--pattern",
    type=str,
    default="*",
    help="文件匹配模式 (默认: *)",
)
def batch(log_dir: str, framework: str, output: str, pattern: str):
    """批量转换目录下的日志文件

    LOG_DIR: 包含日志文件的目录路径
    """
    adapter = _get_adapter(framework)
    if adapter is None:
        click.echo(f"不支持的框架: {framework}", err=True)
        sys.exit(1)

    click.echo("正在批量转换...")
    click.echo(f"  目录: {log_dir}")
    click.echo(f"  框架: {framework}")
    click.echo(f"  模式: {pattern}")

    try:
        from agentrecorder.recorder import Recorder

        recorder = Recorder(adapter)
        trajectories = recorder.convert_batch(log_dir, pattern=pattern)

        output_path = Path(output)
        output_path.parent.mkdir(parents=True, exist_ok=True)

        for trajectory in trajectories:
            trajectory.to_jsonl(output_path)

        click.echo(f"批量转换完成: {output}")
        click.echo(f"  转换文件数: {len(trajectories)}")
    except NotImplementedError as e:
        click.echo(f"适配器尚未实现: {e}", err=True)
        sys.exit(1)
    except Exception as e:
        click.echo(f"转换失败: {e}", err=True)
        sys.exit(1)


def _get_adapter(framework: str):
    """根据框架名称获取适配器实例."""
    if framework == "openhands":
        from agentrecorder.adapters.openhands import OpenHandsAdapter

        return OpenHandsAdapter()
    elif framework == "swe-agent":
        from agentrecorder.adapters.sweagent import SWEAgentAdapter

        return SWEAgentAdapter()
    return None


if __name__ == "__main__":
    main()



"""AgentRecorder MCP Server - Model Context Protocol 服务."""

from pathlib import Path
from typing import Any, Dict, List

try:
    from mcp.server import Server
    from mcp.server.stdio import stdio_server
    from mcp.types import Tool, TextContent

    HAS_MCP = True
except ImportError:
    HAS_MCP = False

from agentrecorder.schema import Trajectory


def create_server() -> "Server":
    """创建 MCP 服务器实例."""
    if not HAS_MCP:
        raise ImportError("MCP 未安装。请运行: pip install knowlyr-recorder[mcp]")

    server = Server("agentrecorder")

    @server.list_tools()
    async def list_tools() -> List[Tool]:
        """列出可用的工具."""
        return [
            Tool(
                name="convert_logs",
                description="将 Agent 日志转换为标准化轨迹格式",
                inputSchema={
                    "type": "object",
                    "properties": {
                        "log_path": {
                            "type": "string",
                            "description": "Agent 日志文件路径",
                        },
                        "framework": {
                            "type": "string",
                            "enum": ["openhands", "swe-agent"],
                            "description": "Agent 框架名称",
                        },
                        "output_path": {
                            "type": "string",
                            "description": "输出 JSONL 文件路径（可选）",
                        },
                    },
                    "required": ["log_path", "framework"],
                },
            ),
            Tool(
                name="validate_logs",
                description="验证日志文件是否为指定的 Agent 框架格式",
                inputSchema={
                    "type": "object",
                    "properties": {
                        "log_path": {
                            "type": "string",
                            "description": "待验证的日志文件路径",
                        },
                        "framework": {
                            "type": "string",
                            "enum": ["openhands", "swe-agent"],
                            "description": "要验证的 Agent 框架名称（可选，不指定则自动检测）",
                        },
                    },
                    "required": ["log_path"],
                },
            ),
            Tool(
                name="get_schema",
                description="返回标准化轨迹的 JSON Schema 定义",
                inputSchema={
                    "type": "object",
                    "properties": {},
                },
            ),
        ]

    @server.call_tool()
    async def call_tool(name: str, arguments: Dict[str, Any]) -> List[TextContent]:
        """调用工具."""

        if name == "convert_logs":
            log_path = arguments["log_path"]
            framework = arguments["framework"]
            output_path = arguments.get("output_path")

            path = Path(log_path)
            if not path.exists():
                return [TextContent(type="text", text=f"日志文件不存在: {log_path}")]

            # 获取适配器
            adapter = _get_adapter(framework)
            if adapter is None:
                return [TextContent(type="text", text=f"不支持的框架: {framework}")]

            if not adapter.validate(log_path):
                return [
                    TextContent(
                        type="text",
                        text=f"日志格式不匹配 {framework} 格式: {log_path}",
                    )
                ]

            try:
                from agentrecorder.recorder import Recorder

                recorder = Recorder(adapter)
                trajectory = recorder.convert(log_path)

                if output_path:
                    trajectory.to_jsonl(output_path)
                    return [
                        TextContent(
                            type="text",
                            text=f"转换成功:\n"
                            f"- 输出路径: {output_path}\n"
                            f"- Agent: {trajectory.agent}\n"
                            f"- 步骤数: {len(trajectory.steps)}\n"
                            f"- 结果: {'成功' if trajectory.outcome.success else '失败'}",
                        )
                    ]
                else:
                    return [
                        TextContent(
                            type="text",
                            text=trajectory.model_dump_json(indent=2),
                        )
                    ]
            except NotImplementedError as e:
                return [TextContent(type="text", text=f"适配器尚未实现: {e}")]
            except Exception as e:
                return [TextContent(type="text", text=f"转换失败: {e}")]

        elif name == "validate_logs":
            log_path = arguments["log_path"]
            framework = arguments.get("framework")

            path = Path(log_path)
            if not path.exists():
                return [TextContent(type="text", text=f"文件不存在: {log_path}")]

            if framework:
                adapter = _get_adapter(framework)
                if adapter is None:
                    return [TextContent(type="text", text=f"不支持的框架: {framework}")]
                is_valid = adapter.validate(log_path)
                return [
                    TextContent(
                        type="text",
                        text=f"验证结果: {'有效' if is_valid else '无效'}\n"
                        f"- 文件: {log_path}\n"
                        f"- 框架: {framework}",
                    )
                ]
            else:
                # 自动检测
                results = []
                for fw_name in ["openhands", "swe-agent"]:
                    adapter = _get_adapter(fw_name)
                    if adapter and adapter.validate(log_path):
                        results.append(fw_name)

                if results:
                    return [
                        TextContent(
                            type="text",
                            text=f"检测到匹配的框架: {', '.join(results)}\n- 文件: {log_path}",
                        )
                    ]
                else:
                    return [
                        TextContent(
                            type="text",
                            text=f"未检测到匹配的框架格式: {log_path}",
                        )
                    ]

        elif name == "get_schema":
            schema_json = Trajectory.schema_json_example()
            return [
                TextContent(
                    type="text",
                    text=f"## Trajectory JSON Schema\n\n```json\n{schema_json}\n```",
                )
            ]

        else:
            return [TextContent(type="text", text=f"未知工具: {name}")]

    return server


def _get_adapter(framework: str):
    """根据框架名称获取适配器实例."""
    if framework == "openhands":
        from agentrecorder.adapters.openhands import OpenHandsAdapter

        return OpenHandsAdapter()
    elif framework == "swe-agent":
        from agentrecorder.adapters.sweagent import SWEAgentAdapter

        return SWEAgentAdapter()
    return None


async def serve():
    """启动 MCP 服务器."""
    if not HAS_MCP:
        raise ImportError("MCP 未安装。请运行: pip install knowlyr-recorder[mcp]")

    server = create_server()
    async with stdio_server() as (read_stream, write_stream):
        await server.run(read_stream, write_stream)


def main():
    """主入口."""
    import asyncio

    asyncio.run(serve())


if __name__ == "__main__":
    main()



"""适配器基类定义."""

from abc import ABC, abstractmethod

from agentrecorder.schema import Trajectory


class BaseAdapter(ABC):
    """Agent 框架适配器基类.

    所有适配器必须继承此类，并实现 parse() 和 validate() 方法。

    实现新适配器时：
    1. 继承 BaseAdapter
    2. 实现 parse() 将日志转换为 Trajectory
    3. 实现 validate() 验证日志格式
    4. 调用 register_adapter() 注册

    Attributes:
        domain: 适配器所属领域 (coding / browser / 等)
    """

    domain: str = "coding"

    @abstractmethod
    def parse(self, log_path: str) -> Trajectory:
        """将 Agent 日志解析为标准轨迹格式.

        Args:
            log_path: 日志文件路径。

        Returns:
            标准化轨迹对象。
        """
        ...

    @abstractmethod
    def validate(self, log_path: str) -> bool:
        """验证日志文件是否为该框架格式.

        Args:
            log_path: 日志文件路径。

        Returns:
            如果日志格式匹配该适配器则返回 True。
        """
        ...



"""适配器单元测试."""

import json

import pytest

from agentrecorder.adapters import OpenHandsAdapter, SWEAgentAdapter


# ── OpenHands 测试数据 ──────────────────────────────────────────────


def _write_openhands_log(path, events):
    """写入 OpenHands JSONL 日志."""
    with open(path, "w", encoding="utf-8") as f:
        for event in events:
            f.write(json.dumps(event, ensure_ascii=False) + "\n")


def _make_openhands_events():
    """构造一个完整的 OpenHands 日志事件序列."""
    return [
        # 用户任务描述
        {
            "id": 0,
            "source": "user",
            "action": "message",
            "args": {"content": "修复 sort.py 的 off-by-one 错误", "task_id": "oh-001"},
            "timestamp": "2026-01-01T10:00:00Z",
        },
        # Agent 读文件
        {
            "id": 1,
            "source": "agent",
            "action": "read",
            "args": {"path": "/workspace/sort.py"},
            "thought": "先看看代码",
            "timestamp": "2026-01-01T10:01:00Z",
            "extras": {"model": "claude-sonnet-4-20250514", "agent_class": "CodeActAgent"},
        },
        {
            "id": 2,
            "observation": "read",
            "content": "def sort(lst):\n    pass\n",
            "extras": {},
        },
        # Agent 执行命令
        {
            "id": 3,
            "source": "agent",
            "action": "run",
            "args": {"command": "pytest tests/"},
            "thought": "运行测试",
            "timestamp": "2026-01-01T10:02:00Z",
            "extras": {"token_count": 100},
        },
        {
            "id": 4,
            "observation": "run",
            "content": "1 passed in 0.02s",
            "extras": {"exit_code": 0},
        },
        # Agent 编辑文件
        {
            "id": 5,
            "source": "agent",
            "action": "edit",
            "args": {"path": "/workspace/sort.py", "old_str": "pass", "new_str": "return sorted(lst)"},
            "thought": "修复排序",
            "timestamp": "2026-01-01T10:03:00Z",
        },
        {
            "id": 6,
            "observation": "edit",
            "content": "File edited successfully",
            "extras": {},
        },
        # 完成
        {
            "id": 7,
            "action": "finish",
            "args": {"outputs": {"test_result": {"success": True, "tests_passed": 3, "tests_failed": 0}}},
        },
    ]


# ── SWE-agent 测试数据 ─────────────────────────────────────────────


def _write_sweagent_log(path, data):
    """写入 SWE-agent JSON 日志."""
    with open(path, "w", encoding="utf-8") as f:
        json.dump(data, f, ensure_ascii=False)


def _make_sweagent_data():
    """构造一个完整的 SWE-agent 轨迹数据."""
    return {
        "environment": "swe_main",
        "problem_statement": "修复 Django model 的 N+1 查询问题",
        "history": [
            [
                {"action": "bash", "command": "find . -name models.py", "thought": "找到模型文件"},
                "./app/models.py\n",
            ],
            [
                {"action": "open", "path": "./app/models.py"},
                "class User(Model):\n    name = CharField()\n",
            ],
            [
                {"action": "edit", "path": "./app/models.py", "old_str": "name", "new_str": "username"},
                "File edited successfully",
            ],
            [
                {"action": "submit"},
                "Submitted",
            ],
        ],
        "info": {
            "instance_id": "django__django-12345",
            "model_name": "gpt-4o",
            "repo": "django/django",
            "base_commit": "abc123",
            "exit_status": "submitted",
            "model_stats": {"tokens_sent": 5000, "tokens_received": 2000},
        },
    }


# ── OpenHandsAdapter 测试 ──────────────────────────────────────────


class TestOpenHandsValidate:
    """验证 OpenHands 日志格式检测."""

    def test_valid_jsonl(self, tmp_path):
        """合法的 OpenHands JSONL 应通过验证."""
        path = tmp_path / "log.jsonl"
        _write_openhands_log(path, [{"action": "message", "source": "user"}])
        assert OpenHandsAdapter().validate(str(path)) is True

    def test_invalid_extension(self, tmp_path):
        """非 .jsonl/.json 文件应失败."""
        path = tmp_path / "log.txt"
        path.write_text('{"action": "run"}')
        assert OpenHandsAdapter().validate(str(path)) is False

    def test_empty_file(self, tmp_path):
        """空文件应失败."""
        path = tmp_path / "log.jsonl"
        path.write_text("")
        assert OpenHandsAdapter().validate(str(path)) is False

    def test_no_action_field(self, tmp_path):
        """没有 action/observation 字段应失败."""
        path = tmp_path / "log.jsonl"
        path.write_text('{"type": "other"}\n')
        assert OpenHandsAdapter().validate(str(path)) is False

    def test_nonexistent_file(self):
        """不存在的文件应失败."""
        assert OpenHandsAdapter().validate("/nonexistent/log.jsonl") is False


class TestOpenHandsParse:
    """验证 OpenHands 日志解析."""

    def test_full_parse(self, tmp_path):
        """完整日志应正确解析为 Trajectory."""
        path = tmp_path / "log.jsonl"
        _write_openhands_log(path, _make_openhands_events())

        adapter = OpenHandsAdapter()
        traj = adapter.parse(str(path))

        assert traj.agent == "openhands/CodeActAgent"
        assert traj.model == "claude-sonnet-4-20250514"
        assert traj.task.task_id == "oh-001"
        assert "off-by-one" in traj.task.description
        assert len(traj.steps) == 3  # read + run + edit

    def test_step_mapping(self, tmp_path):
        """action 类型应正确映射到工具名."""
        path = tmp_path / "log.jsonl"
        _write_openhands_log(path, _make_openhands_events())

        traj = OpenHandsAdapter().parse(str(path))
        tool_names = [s.tool_call.name for s in traj.steps]
        assert tool_names == ["read_file", "bash", "edit_file"]

    def test_tool_result(self, tmp_path):
        """observation 应正确映射到 ToolResult."""
        path = tmp_path / "log.jsonl"
        _write_openhands_log(path, _make_openhands_events())

        traj = OpenHandsAdapter().parse(str(path))
        # bash 命令的结果
        bash_step = traj.steps[1]
        assert bash_step.tool_result.output == "1 passed in 0.02s"
        assert bash_step.tool_result.exit_code == 0
        assert bash_step.tool_result.success is True

    def test_outcome(self, tmp_path):
        """finish 事件应正确设置 outcome."""
        path = tmp_path / "log.jsonl"
        _write_openhands_log(path, _make_openhands_events())

        traj = OpenHandsAdapter().parse(str(path))
        assert traj.outcome.success is True
        assert traj.outcome.tests_passed == 3
        assert traj.outcome.tests_failed == 0
        assert traj.outcome.total_steps == 3

    def test_thought_preserved(self, tmp_path):
        """agent 的 thought 应保留."""
        path = tmp_path / "log.jsonl"
        _write_openhands_log(path, _make_openhands_events())

        traj = OpenHandsAdapter().parse(str(path))
        assert traj.steps[0].thought == "先看看代码"
        assert traj.steps[1].thought == "运行测试"

    def test_token_count(self, tmp_path):
        """token_count 应从 extras 中提取."""
        path = tmp_path / "log.jsonl"
        _write_openhands_log(path, _make_openhands_events())

        traj = OpenHandsAdapter().parse(str(path))
        assert traj.steps[1].token_count == 100
        assert traj.steps[0].token_count is None  # 没有 token_count

    def test_skip_user_messages(self, tmp_path):
        """用户消息不应生成 Step."""
        path = tmp_path / "log.jsonl"
        _write_openhands_log(path, _make_openhands_events())

        traj = OpenHandsAdapter().parse(str(path))
        # 只有 3 个 agent 操作，不含用户消息
        assert len(traj.steps) == 3

    def test_empty_log(self, tmp_path):
        """空日志应返回空步骤."""
        path = tmp_path / "log.jsonl"
        _write_openhands_log(path, [])

        traj = OpenHandsAdapter().parse(str(path))
        assert len(traj.steps) == 0
        assert traj.outcome.success is False

    def test_metadata_extraction(self, tmp_path):
        """extras 中的元数据应被提取."""
        path = tmp_path / "log.jsonl"
        events = _make_openhands_events()
        events[1]["extras"]["instance_id"] = "swebench-123"
        events[1]["extras"]["max_iterations"] = 30
        _write_openhands_log(path, events)

        traj = OpenHandsAdapter().parse(str(path))
        assert traj.metadata["instance_id"] == "swebench-123"
        assert traj.metadata["max_iterations"] == 30
        assert traj.metadata["agent_class"] == "CodeActAgent"

    def test_error_observation(self, tmp_path):
        """含 error 的 observation 应设置 ToolResult.error."""
        path = tmp_path / "log.jsonl"
        events = [
            {"id": 1, "source": "agent", "action": "run", "args": {"command": "bad"},
             "thought": "试试", "timestamp": "2026-01-01T10:00:00Z"},
            {"id": 2, "observation": "run", "content": "command not found",
             "extras": {"exit_code": 127, "error": "command not found"}},
        ]
        _write_openhands_log(path, events)

        traj = OpenHandsAdapter().parse(str(path))
        assert traj.steps[0].tool_result.exit_code == 127
        assert traj.steps[0].tool_result.error == "command not found"
        assert traj.steps[0].tool_result.success is False


# ── SWEAgentAdapter 测试 ───────────────────────────────────────────


class TestSWEAgentValidate:
    """验证 SWE-agent 日志格式检测."""

    def test_valid_json(self, tmp_path):
        """合法的 SWE-agent JSON 应通过验证."""
        path = tmp_path / "traj.json"
        _write_sweagent_log(path, _make_sweagent_data())
        assert SWEAgentAdapter().validate(str(path)) is True

    def test_invalid_extension(self, tmp_path):
        """非 .json 文件应失败."""
        path = tmp_path / "traj.jsonl"
        path.write_text('{"history": [], "info": {}}')
        assert SWEAgentAdapter().validate(str(path)) is False

    def test_missing_fields(self, tmp_path):
        """缺少 history/info 字段应失败."""
        path = tmp_path / "traj.json"
        _write_sweagent_log(path, {"data": "other"})
        assert SWEAgentAdapter().validate(str(path)) is False

    def test_nonexistent_file(self):
        """不存在的文件应失败."""
        assert SWEAgentAdapter().validate("/nonexistent/traj.json") is False


class TestSWEAgentParse:
    """验证 SWE-agent 日志解析."""

    def test_full_parse(self, tmp_path):
        """完整日志应正确解析为 Trajectory."""
        path = tmp_path / "traj.json"
        _write_sweagent_log(path, _make_sweagent_data())

        traj = SWEAgentAdapter().parse(str(path))

        assert traj.agent == "swe-agent"
        assert traj.model == "gpt-4o"
        assert traj.task.task_id == "django__django-12345"
        assert "N+1" in traj.task.description
        assert len(traj.steps) == 4  # bash + open + edit + submit

    def test_task_info(self, tmp_path):
        """task 字段应从 info 中正确提取."""
        path = tmp_path / "traj.json"
        _write_sweagent_log(path, _make_sweagent_data())

        traj = SWEAgentAdapter().parse(str(path))
        assert traj.task.repo == "django/django"
        assert traj.task.base_commit == "abc123"

    def test_step_mapping(self, tmp_path):
        """action 类型应正确映射到工具名."""
        path = tmp_path / "traj.json"
        _write_sweagent_log(path, _make_sweagent_data())

        traj = SWEAgentAdapter().parse(str(path))
        tool_names = [s.tool_call.name for s in traj.steps]
        assert tool_names == ["bash", "read_file", "edit_file", "submit"]

    def test_outcome_submitted(self, tmp_path):
        """exit_status=submitted 应设为成功."""
        path = tmp_path / "traj.json"
        _write_sweagent_log(path, _make_sweagent_data())

        traj = SWEAgentAdapter().parse(str(path))
        assert traj.outcome.success is True
        assert traj.outcome.total_steps == 4

    def test_outcome_failed(self, tmp_path):
        """非 submitted 的 exit_status 应设为失败."""
        path = tmp_path / "traj.json"
        data = _make_sweagent_data()
        data["info"]["exit_status"] = "error"
        _write_sweagent_log(path, data)

        traj = SWEAgentAdapter().parse(str(path))
        assert traj.outcome.success is False

    def test_token_stats(self, tmp_path):
        """model_stats 应正确计算 total_tokens."""
        path = tmp_path / "traj.json"
        _write_sweagent_log(path, _make_sweagent_data())

        traj = SWEAgentAdapter().parse(str(path))
        assert traj.outcome.total_tokens == 7000  # 5000 + 2000

    def test_thought_preserved(self, tmp_path):
        """action 中的 thought 应保留."""
        path = tmp_path / "traj.json"
        _write_sweagent_log(path, _make_sweagent_data())

        traj = SWEAgentAdapter().parse(str(path))
        assert traj.steps[0].thought == "找到模型文件"

    def test_metadata_extraction(self, tmp_path):
        """元数据应从 info 中正确提取."""
        path = tmp_path / "traj.json"
        _write_sweagent_log(path, _make_sweagent_data())

        traj = SWEAgentAdapter().parse(str(path))
        assert traj.metadata["instance_id"] == "django__django-12345"
        assert traj.metadata["exit_status"] == "submitted"
        assert traj.metadata["environment"] == "swe_main"
        assert traj.metadata["model_stats"]["tokens_sent"] == 5000

    def test_empty_history(self, tmp_path):
        """空 history 应返回空步骤."""
        path = tmp_path / "traj.json"
        data = _make_sweagent_data()
        data["history"] = []
        _write_sweagent_log(path, data)

        traj = SWEAgentAdapter().parse(str(path))
        assert len(traj.steps) == 0

    def test_string_action(self, tmp_path):
        """纯字符串 action 应作为 bash 命令处理."""
        path = tmp_path / "traj.json"
        data = _make_sweagent_data()
        data["history"] = [["ls -la", "file1\nfile2\n"]]
        _write_sweagent_log(path, data)

        traj = SWEAgentAdapter().parse(str(path))
        assert len(traj.steps) == 1
        assert traj.steps[0].tool_call.name == "bash"
        assert traj.steps[0].tool_call.parameters == {"command": "ls -la"}

    def test_skip_think_actions(self, tmp_path):
        """think action 应被跳过."""
        path = tmp_path / "traj.json"
        data = _make_sweagent_data()
        data["history"] = [
            [{"action": "think", "output": "让我想想..."}, ""],
            [{"action": "bash", "command": "ls"}, "file.py\n"],
        ]
        _write_sweagent_log(path, data)

        traj = SWEAgentAdapter().parse(str(path))
        assert len(traj.steps) == 1
        assert traj.steps[0].tool_call.name == "bash"


# ── Recorder + Adapter 集成 ────────────────────────────────────────


class TestRecorderWithAdapters:
    """验证 Recorder 与适配器的集成."""

    def test_recorder_convert_openhands(self, tmp_path):
        """Recorder.convert 应使用 OpenHands 适配器."""
        from agentrecorder import Recorder

        path = tmp_path / "log.jsonl"
        _write_openhands_log(path, _make_openhands_events())

        recorder = Recorder(OpenHandsAdapter())
        traj = recorder.convert(str(path))
        assert len(traj.steps) == 3
        assert traj.agent == "openhands/CodeActAgent"

    def test_recorder_convert_sweagent(self, tmp_path):
        """Recorder.convert 应使用 SWE-agent 适配器."""
        from agentrecorder import Recorder

        path = tmp_path / "traj.json"
        _write_sweagent_log(path, _make_sweagent_data())

        recorder = Recorder(SWEAgentAdapter())
        traj = recorder.convert(str(path))
        assert len(traj.steps) == 4
        assert traj.agent == "swe-agent"

    def test_recorder_batch_convert(self, tmp_path):
        """Recorder.convert_batch 应批量转换."""
        from agentrecorder import Recorder

        for i in range(3):
            path = tmp_path / f"traj_{i}.json"
            _write_sweagent_log(path, _make_sweagent_data())

        recorder = Recorder(SWEAgentAdapter())
        trajectories = recorder.convert_batch(str(tmp_path), "*.json")
        assert len(trajectories) == 3

    def test_recorder_invalid_format(self, tmp_path):
        """格式不匹配应抛出 ValueError."""
        from agentrecorder import Recorder

        path = tmp_path / "bad.jsonl"
        path.write_text('{"type": "not_openhands"}\n')

        recorder = Recorder(OpenHandsAdapter())
        with pytest.raises(ValueError, match="不匹配"):
            recorder.convert(str(path))

    def test_recorder_file_not_found(self):
        """文件不存在应抛出 FileNotFoundError."""
        from agentrecorder import Recorder

        recorder = Recorder(OpenHandsAdapter())
        with pytest.raises(FileNotFoundError):
            recorder.convert("/nonexistent/log.jsonl")

    def test_trajectory_roundtrip(self, tmp_path):
        """解析后的 Trajectory 应可序列化/反序列化."""
        from agentrecorder.schema import Trajectory

        path = tmp_path / "log.jsonl"
        _write_openhands_log(path, _make_openhands_events())

        traj = OpenHandsAdapter().parse(str(path))

        # 序列化
        out_path = tmp_path / "output.jsonl"
        traj.to_jsonl(str(out_path))

        # 反序列化
        loaded = Trajectory.from_jsonl(str(out_path))
        assert len(loaded) == 1
        assert loaded[0].task.task_id == traj.task.task_id
        assert len(loaded[0].steps) == len(traj.steps)



"""核心录制器 - 将 Agent 日志转换为标准化轨迹."""

import logging
from pathlib import Path

from agentrecorder.adapters.base import BaseAdapter
from agentrecorder.schema import Trajectory

logger = logging.getLogger(__name__)


class Recorder:
    """Agent 轨迹录制器.

    使用适配器模式，将不同 Agent 框架的日志转换为标准化轨迹格式。

    Args:
        adapter: Agent 框架适配器实例。

    Example:
        >>> from agentrecorder import Recorder
        >>> from agentrecorder.adapters import OpenHandsAdapter
        >>> recorder = Recorder(OpenHandsAdapter())
        >>> trajectory = recorder.convert("path/to/log.jsonl")
    """

    def __init__(self, adapter: BaseAdapter) -> None:
        self.adapter = adapter

    def convert(self, log_path: str | Path) -> Trajectory:
        """将单个日志文件转换为标准轨迹.

        Args:
            log_path: Agent 日志文件路径。

        Returns:
            标准化轨迹对象。

        Raises:
            FileNotFoundError: 日志文件不存在。
            ValueError: 日志格式不匹配当前适配器。
        """
        log_path = Path(log_path)
        if not log_path.exists():
            raise FileNotFoundError(f"日志文件不存在: {log_path}")

        adapter_name = self.adapter.__class__.__name__
        logger.info("转换日志: %s (适配器: %s)", log_path, adapter_name)

        if not self.adapter.validate(str(log_path)):
            raise ValueError(
                f"日志格式不匹配适配器 {adapter_name}: {log_path}"
            )

        trajectory = self.adapter.parse(str(log_path))
        logger.info("转换完成: %d 步", len(trajectory.steps))
        return trajectory

    def convert_batch(self, log_dir: str | Path, pattern: str = "*") -> list[Trajectory]:
        """批量转换目录下的日志文件.

        Args:
            log_dir: 包含日志文件的目录路径。
            pattern: 文件匹配模式 (默认匹配所有文件)。

        Returns:
            标准化轨迹列表。
        """
        log_dir = Path(log_dir)
        if not log_dir.is_dir():
            raise NotADirectoryError(f"目录不存在: {log_dir}")

        trajectories = []
        for log_path in sorted(log_dir.glob(pattern)):
            if log_path.is_file() and self.adapter.validate(str(log_path)):
                trajectory = self.adapter.parse(str(log_path))
                trajectories.append(trajectory)

        logger.info("批量转换完成: %d 个文件 -> %d 条轨迹", len(list(log_dir.glob(pattern))), len(trajectories))
        return trajectories

    def record(self, sandbox: object, agent: object) -> Trajectory:
        """实时录制 Agent 执行过程 (暂未实现).

        此功能计划支持直接接入 Agent 的执行过程，实时录制轨迹，
        而不是事后从日志解析。

        Args:
            sandbox: 沙箱环境实例。
            agent: Agent 实例。

        Returns:
            录制的标准化轨迹。

        Raises:
            NotImplementedError: 此功能尚未实现。
        """
        raise NotImplementedError(
            "实时录制功能尚未实现。请使用 convert() 从日志文件转换。"
        )



"""OpenHands 适配器.

将 OpenHands (原 OpenDevin) 框架的执行日志转换为标准轨迹格式。

OpenHands 日志格式说明:
- 日志文件通常为 JSONL 格式，每行一个 JSON 对象
- 每个事件包含 action/observation 对
- action 类型包括: CmdRunAction, FileWriteAction, BrowseURLAction 等
- observation 类型包括: CmdOutputObservation, FileReadObservation 等
- 元数据包含 model, agent_class, max_iterations 等
"""

import json
import logging
from datetime import datetime, timezone
from pathlib import Path

from agentrecorder.adapters.base import BaseAdapter
from agentrecorder.schema import Outcome, Step, ToolCall, Trajectory
from knowlyrcore import TaskInfo, ToolResult

logger = logging.getLogger(__name__)

# OpenHands action 类型到标准工具名的映射
_ACTION_TOOL_MAP = {
    "run": "bash",
    "run_ipython": "ipython",
    "read": "read_file",
    "write": "write_file",
    "browse": "browse_url",
    "edit": "edit_file",
    "think": "think",
}


class OpenHandsAdapter(BaseAdapter):
    """OpenHands 框架适配器.

    支持解析 OpenHands 的 JSONL 格式日志文件。

    Example:
        >>> adapter = OpenHandsAdapter()
        >>> if adapter.validate("output/log.jsonl"):
        ...     trajectory = adapter.parse("output/log.jsonl")
    """

    def parse(self, log_path: str) -> Trajectory:
        """将 OpenHands 日志解析为标准轨迹格式.

        OpenHands 日志结构:
        - 每行是一个事件 JSON
        - 事件包含 "action" 或 "observation" 字段
        - action 事件: {"id": N, "action": "run", "args": {"command": "..."}, ...}
        - observation 事件: {"id": N, "observation": "run", "content": "...", ...}

        Args:
            log_path: OpenHands 日志文件路径 (JSONL 格式)。

        Returns:
            标准化轨迹对象。
        """
        path = Path(log_path)
        events = self._read_events(path)

        task_info = self._extract_task(events)
        model, agent = self._extract_metadata(events)
        steps = self._pair_events_to_steps(events)
        outcome = self._determine_outcome(events, steps)
        metadata = self._extract_extra_metadata(events)

        return Trajectory(
            task=task_info,
            agent=agent,
            model=model,
            steps=steps,
            outcome=outcome,
            metadata=metadata,
        )

    def validate(self, log_path: str) -> bool:
        """验证是否为 OpenHands 日志格式.

        检查规则:
        1. 文件必须是 .jsonl 格式
        2. 首行 JSON 应包含 OpenHands 特征字段

        Args:
            log_path: 待验证的文件路径。

        Returns:
            如果是 OpenHands 日志格式则返回 True。
        """
        path = Path(log_path)
        if not path.exists() or path.suffix not in (".jsonl", ".json"):
            return False

        try:
            with open(path, "r", encoding="utf-8") as f:
                first_line = f.readline().strip()
                if not first_line:
                    return False
                data = json.loads(first_line)
                # OpenHands 日志通常包含 action 或 observation 字段
                return "action" in data or "observation" in data
        except (json.JSONDecodeError, OSError):
            return False

    def _read_events(self, path: Path) -> list[dict]:
        """读取 JSONL 事件列表."""
        events = []
        with open(path, "r", encoding="utf-8") as f:
            for line in f:
                line = line.strip()
                if line:
                    events.append(json.loads(line))
        return events

    def _extract_task(self, events: list[dict]) -> TaskInfo:
        """从事件中提取任务信息.

        OpenHands 通常将任务描述放在第一个 message_action 事件中。
        """
        for event in events:
            if event.get("action") == "message" and event.get("source") == "user":
                args = event.get("args", {})
                content = args.get("content", "")
                return TaskInfo(
                    task_id=event.get("args", {}).get("task_id", ""),
                    description=content,
                    metadata={"source": "openhands"},
                )
        return TaskInfo(metadata={"source": "openhands"})

    def _extract_metadata(self, events: list[dict]) -> tuple[str, str]:
        """提取 model 和 agent 信息."""
        model = ""
        agent = "openhands"
        for event in events:
            extras = event.get("extras", {})
            if extras.get("model"):
                model = extras["model"]
            if extras.get("agent_class"):
                agent = f"openhands/{extras['agent_class']}"
            # 有些日志在顶层放 metadata
            if event.get("model"):
                model = event["model"]
        return model, agent

    def _pair_events_to_steps(self, events: list[dict]) -> list[Step]:
        """将 action/observation 事件配对为 Step.

        跳过 message_action（用户输入）和 agent_finish 等非工具事件。
        """
        steps = []
        step_id = 0
        pending_action = None

        for event in events:
            if "action" in event:
                action_type = event["action"]
                source = event.get("source", "")

                # 跳过用户消息和 finish 事件
                if source == "user" or action_type in ("message", "finish"):
                    # 如果前一个 action 未配对，丢弃它
                    pending_action = None
                    continue

                # 记录 agent 的 action，等待后续 observation 配对
                pending_action = event

            elif "observation" in event and pending_action is not None:
                # 配对当前 observation 与上一个 action
                step = self._create_step(step_id, pending_action, event)
                steps.append(step)
                step_id += 1
                pending_action = None

        return steps

    def _create_step(self, step_id: int, action: dict, observation: dict) -> Step:
        """从 action + observation 创建 Step."""
        action_type = action.get("action", "unknown")
        tool_name = _ACTION_TOOL_MAP.get(action_type, action_type)
        args = action.get("args", {})
        thought = action.get("thought", "")

        # 提取 observation 内容
        content = observation.get("content", "")
        extras = observation.get("extras", {})
        exit_code = extras.get("exit_code", 0)
        error = extras.get("error", None)

        # 时间戳
        timestamp = action.get("timestamp", "")
        if not timestamp:
            timestamp = datetime.now(timezone.utc).isoformat()

        return Step(
            step_id=step_id,
            thought=thought,
            tool_call=ToolCall(name=tool_name, parameters=args),
            tool_result=ToolResult(
                output=content,
                exit_code=exit_code if isinstance(exit_code, int) else 0,
                error=error,
            ),
            timestamp=timestamp,
            token_count=action.get("extras", {}).get("token_count"),
        )

    def _determine_outcome(self, events: list[dict], steps: list[Step]) -> Outcome:
        """从事件中判断执行结果."""
        success = False
        tests_passed = 0
        tests_failed = 0

        for event in events:
            # finish 事件通常包含结果
            if event.get("action") == "finish":
                args = event.get("args", {})
                outputs = args.get("outputs", {})
                if "test_result" in outputs:
                    test_result = outputs["test_result"]
                    success = test_result.get("success", False)
                    tests_passed = test_result.get("tests_passed", 0)
                    tests_failed = test_result.get("tests_failed", 0)
                elif outputs.get("success") is not None:
                    success = bool(outputs["success"])
                break

            # 检查最后的测试 observation
            if event.get("observation") == "run":
                content = event.get("content", "")
                exit_code = event.get("extras", {}).get("exit_code", -1)
                if exit_code == 0 and ("passed" in content or "PASSED" in content):
                    success = True

        total_tokens = sum(
            s.token_count for s in steps if s.token_count is not None
        )

        return Outcome(
            success=success,
            tests_passed=tests_passed,
            tests_failed=tests_failed,
            total_steps=len(steps),
            total_tokens=total_tokens,
        )

    def _extract_extra_metadata(self, events: list[dict]) -> dict:
        """提取额外元数据."""
        metadata = {}
        for event in events:
            extras = event.get("extras", {})
            if extras.get("instance_id"):
                metadata["instance_id"] = extras["instance_id"]
            if extras.get("max_iterations"):
                metadata["max_iterations"] = extras["max_iterations"]
            if extras.get("agent_class"):
                metadata["agent_class"] = extras["agent_class"]
        return metadata



"""测试 Recorder CLI 命令."""

import json

from click.testing import CliRunner

from agentrecorder.cli import main


def _write_openhands_log(path):
    """写最小的 OpenHands JSONL 日志."""
    events = [
        {
            "id": 0, "source": "user", "action": "message",
            "args": {"content": "修复 bug", "task_id": "oh-cli"},
            "timestamp": "2026-01-01T10:00:00Z",
        },
        {
            "id": 1, "source": "agent", "action": "run",
            "args": {"command": "pytest"},
            "thought": "跑测试",
            "timestamp": "2026-01-01T10:01:00Z",
            "extras": {"model": "claude-sonnet-4-20250514", "agent_class": "CodeActAgent"},
        },
        {
            "id": 2, "observation": "run",
            "content": "1 passed",
            "extras": {"exit_code": 0},
        },
        {
            "id": 3, "action": "finish",
            "args": {"outputs": {"test_result": {
                "success": True, "tests_passed": 1, "tests_failed": 0,
            }}},
        },
    ]
    with open(path, "w") as f:
        for e in events:
            f.write(json.dumps(e) + "\n")


def _write_sweagent_log(path):
    """写最小的 SWE-agent JSON 日志."""
    data = {
        "problem_statement": "修复 Django bug",
        "history": [
            [{"action": "bash", "command": "ls", "thought": "看看"}, "file.py\n"],
            [{"action": "submit"}, "Submitted"],
        ],
        "info": {
            "instance_id": "swe-001",
            "model_name": "gpt-4o",
            "exit_status": "submitted",
        },
    }
    with open(path, "w") as f:
        json.dump(data, f)


class TestConvertCommand:
    """测试 convert 命令."""

    def test_convert_openhands(self, tmp_path):
        """转换 OpenHands 日志."""
        log_path = tmp_path / "log.jsonl"
        _write_openhands_log(log_path)

        runner = CliRunner()
        result = runner.invoke(main, ["convert", str(log_path), "-f", "openhands"])

        assert result.exit_code == 0
        assert "oh-cli" in result.output or "转换" in result.output

    def test_convert_sweagent(self, tmp_path):
        """转换 SWE-agent 日志."""
        log_path = tmp_path / "traj.json"
        _write_sweagent_log(log_path)

        runner = CliRunner()
        result = runner.invoke(main, ["convert", str(log_path), "-f", "swe-agent"])

        assert result.exit_code == 0

    def test_convert_with_output(self, tmp_path):
        """使用 -o 输出到文件."""
        log_path = tmp_path / "log.jsonl"
        _write_openhands_log(log_path)
        out_path = tmp_path / "output.jsonl"

        runner = CliRunner()
        result = runner.invoke(main, [
            "convert", str(log_path), "-f", "openhands", "-o", str(out_path),
        ])

        assert result.exit_code == 0
        assert "转换成功" in result.output


class TestValidateCommand:
    """测试 validate 命令."""

    def test_validate_openhands(self, tmp_path):
        """验证 OpenHands 日志."""
        log_path = tmp_path / "log.jsonl"
        _write_openhands_log(log_path)

        runner = CliRunner()
        result = runner.invoke(main, ["validate", str(log_path), "-f", "openhands"])

        assert result.exit_code == 0
        assert "有效" in result.output

    def test_validate_auto_detect(self, tmp_path):
        """自动检测框架."""
        log_path = tmp_path / "log.jsonl"
        _write_openhands_log(log_path)

        runner = CliRunner()
        result = runner.invoke(main, ["validate", str(log_path)])

        assert result.exit_code == 0
        assert "匹配框架" in result.output

    def test_validate_wrong_format(self, tmp_path):
        """格式不匹配应报错."""
        log_path = tmp_path / "bad.jsonl"
        with open(log_path, "w") as f:
            f.write("not a valid log\n")

        runner = CliRunner()
        result = runner.invoke(main, ["validate", str(log_path), "-f", "openhands"])

        assert result.exit_code != 0


class TestSchemaCommand:
    """测试 schema 命令."""

    def test_schema_output(self):
        """schema 命令应输出内容."""
        runner = CliRunner()
        result = runner.invoke(main, ["schema"])

        assert result.exit_code == 0
        assert len(result.output) > 0


class TestBatchCommand:
    """测试 batch 命令."""

    def test_batch_sweagent(self, tmp_path):
        """批量转换 SWE-agent 日志."""
        log_dir = tmp_path / "logs"
        log_dir.mkdir()
        for i in range(2):
            _write_sweagent_log(log_dir / f"traj_{i}.json")

        out_path = tmp_path / "output.jsonl"

        runner = CliRunner()
        result = runner.invoke(main, [
            "batch", str(log_dir),
            "-f", "swe-agent",
            "-o", str(out_path),
            "-p", "*.json",
        ])

        assert result.exit_code == 0
        assert "批量转换完成" in result.output
        assert "转换文件数: 2" in result.output



"""标准化轨迹 Schema 定义.

定义 Agent 执行轨迹的数据模型，所有适配器将日志转换为此格式。
"""

from __future__ import annotations

import json
from pathlib import Path
from typing import Any

from pydantic import BaseModel, Field

from knowlyrcore import TaskInfo, ToolResult


class ToolCall(BaseModel):
    """工具调用记录."""

    name: str = Field(description="工具名称")
    parameters: dict[str, Any] = Field(default_factory=dict, description="工具调用参数")


class Step(BaseModel):
    """单步执行记录."""

    step_id: int = Field(description="步骤编号")
    thought: str = Field(description="Agent 思考过程")
    tool_call: ToolCall = Field(description="工具调用")
    tool_result: ToolResult = Field(description="工具执行结果")
    timestamp: str = Field(description="时间戳 (ISO 8601)")
    token_count: int | None = Field(default=None, description="该步骤消耗的 Token 数")


class Outcome(BaseModel):
    """执行结果.

    tests_passed/tests_failed 为 coding 领域字段，保留向后兼容。
    非 coding 领域使用 score + outcome_metadata 存放结果数据。
    """

    success: bool = Field(description="是否成功")
    score: float = Field(default=0.0, description="归一化分数 [0.0, 1.0]")
    tests_passed: int = Field(default=0, description="通过的测试数 (coding)")
    tests_failed: int = Field(default=0, description="失败的测试数 (coding)")
    total_steps: int = Field(default=0, description="总步骤数")
    total_tokens: int = Field(default=0, description="总 Token 消耗")
    outcome_metadata: dict[str, Any] = Field(
        default_factory=dict, description="领域相关结果数据"
    )


class Trajectory(BaseModel):
    """标准化 Agent 执行轨迹.

    这是 AgentRecorder 的核心数据模型，所有适配器都将日志转换为此格式。
    """

    task: TaskInfo = Field(description="任务信息")
    agent: str = Field(description="Agent 框架名称 (如 openhands, swe-agent)")
    model: str = Field(default="", description="使用的 LLM 模型")
    steps: list[Step] = Field(default_factory=list, description="执行步骤列表")
    outcome: Outcome = Field(description="执行结果")
    metadata: dict[str, Any] = Field(default_factory=dict, description="额外元数据")

    def to_jsonl(self, path: str | Path) -> None:
        """将轨迹保存为 JSONL 格式.

        Args:
            path: 输出文件路径。如果文件已存在，将追加写入。
        """
        path = Path(path)
        path.parent.mkdir(parents=True, exist_ok=True)
        with open(path, "a", encoding="utf-8") as f:
            f.write(self.model_dump_json() + "\n")

    @classmethod
    def from_jsonl(cls, path: str | Path) -> list[Trajectory]:
        """从 JSONL 文件加载轨迹列表.

        Args:
            path: JSONL 文件路径，每行一个 JSON 对象。

        Returns:
            轨迹对象列表。
        """
        path = Path(path)
        trajectories = []
        with open(path, "r", encoding="utf-8") as f:
            for line in f:
                line = line.strip()
                if line:
                    data = json.loads(line)
                    trajectories.append(cls.model_validate(data))
        return trajectories

    @classmethod
    def schema_json_example(cls) -> str:
        """返回 JSON Schema 的格式化字符串."""
        return json.dumps(cls.model_json_schema(), indent=2, ensure_ascii=False)



"""测试 Schema 数据模型."""

import json
import tempfile
from pathlib import Path


from agentrecorder.schema import (
    Trajectory,
    Step,
    ToolCall,
    ToolResult,
    TaskInfo,
    Outcome,
)


class TestToolCall:
    """ToolCall 模型测试."""

    def test_create(self):
        tc = ToolCall(name="bash", parameters={"command": "ls -la"})
        assert tc.name == "bash"
        assert tc.parameters == {"command": "ls -la"}

    def test_empty_parameters(self):
        tc = ToolCall(name="submit", parameters={})
        assert tc.name == "submit"
        assert tc.parameters == {}

    def test_default_parameters(self):
        tc = ToolCall(name="submit")
        assert tc.parameters == {}

    def test_serialization(self):
        tc = ToolCall(name="edit", parameters={"file": "main.py", "content": "print('hi')"})
        data = tc.model_dump()
        assert data["name"] == "edit"
        assert data["parameters"]["file"] == "main.py"


class TestToolResult:
    """ToolResult 模型测试."""

    def test_create_success(self):
        tr = ToolResult(output="file1.py\nfile2.py", exit_code=0)
        assert tr.output == "file1.py\nfile2.py"
        assert tr.exit_code == 0
        assert tr.error is None

    def test_create_error(self):
        tr = ToolResult(output="", exit_code=1, error="command not found")
        assert tr.exit_code == 1
        assert tr.error == "command not found"

    def test_default_exit_code(self):
        tr = ToolResult(output="ok")
        assert tr.exit_code == 0

    def test_serialization(self):
        tr = ToolResult(output="done", exit_code=0, error=None)
        data = tr.model_dump()
        assert data["output"] == "done"
        assert data["exit_code"] == 0
        assert data["error"] is None


class TestStep:
    """Step 模型测试."""

    def test_create(self):
        step = Step(
            step_id=1,
            thought="我需要查看文件列表",
            tool_call=ToolCall(name="bash", parameters={"command": "ls"}),
            tool_result=ToolResult(output="main.py\ntest.py", exit_code=0),
            timestamp="2026-01-15T10:30:00Z",
            token_count=150,
        )
        assert step.step_id == 1
        assert step.thought == "我需要查看文件列表"
        assert step.tool_call.name == "bash"
        assert step.tool_result.exit_code == 0
        assert step.token_count == 150

    def test_optional_token_count(self):
        step = Step(
            step_id=1,
            thought="思考",
            tool_call=ToolCall(name="bash", parameters={}),
            tool_result=ToolResult(output="ok"),
            timestamp="2026-01-15T10:30:00Z",
        )
        assert step.token_count is None

    def test_serialization(self):
        step = Step(
            step_id=1,
            thought="查看文件",
            tool_call=ToolCall(name="bash", parameters={"command": "ls"}),
            tool_result=ToolResult(output="file.py"),
            timestamp="2026-01-15T10:30:00Z",
        )
        data = step.model_dump()
        assert data["step_id"] == 1
        assert data["tool_call"]["name"] == "bash"


class TestTrajectory:
    """Trajectory 模型测试."""

    def _make_trajectory(self) -> Trajectory:
        """创建一个测试用的 Trajectory."""
        return Trajectory(
            task=TaskInfo(
                task_id="test-001",
                description="修复排序 bug",
                type="bug_fix",
                language="python",
                difficulty="easy",
                repo="owner/repo",
                base_commit="abc123",
                test_command="pytest tests/",
            ),
            agent="openhands",
            model="claude-sonnet-4-20250514",
            steps=[
                Step(
                    step_id=1,
                    thought="先看看代码",
                    tool_call=ToolCall(name="bash", parameters={"command": "cat sort.py"}),
                    tool_result=ToolResult(output="def sort(lst):\n    return lst"),
                    timestamp="2026-01-15T10:30:00Z",
                    token_count=100,
                ),
                Step(
                    step_id=2,
                    thought="修复排序实现",
                    tool_call=ToolCall(
                        name="edit",
                        parameters={"file": "sort.py", "content": "def sort(lst):\n    return sorted(lst)"},
                    ),
                    tool_result=ToolResult(output="File edited successfully"),
                    timestamp="2026-01-15T10:31:00Z",
                    token_count=120,
                ),
            ],
            outcome=Outcome(
                success=True,
                tests_passed=5,
                tests_failed=0,
                total_steps=2,
                total_tokens=220,
            ),
            metadata={"run_id": "run-abc", "duration": 42.5},
        )

    def test_create(self):
        traj = self._make_trajectory()
        assert traj.agent == "openhands"
        assert traj.model == "claude-sonnet-4-20250514"
        assert len(traj.steps) == 2
        assert traj.outcome.success is True
        assert traj.task.task_id == "test-001"

    def test_serialization(self):
        traj = self._make_trajectory()
        json_str = traj.model_dump_json()
        data = json.loads(json_str)
        assert data["agent"] == "openhands"
        assert len(data["steps"]) == 2
        assert data["outcome"]["success"] is True

    def test_to_jsonl_and_from_jsonl(self):
        traj = self._make_trajectory()

        with tempfile.TemporaryDirectory() as tmpdir:
            path = Path(tmpdir) / "trajectories.jsonl"
            traj.to_jsonl(path)

            # 验证文件存在且非空
            assert path.exists()
            content = path.read_text()
            assert content.strip()

            # 从 JSONL 加载
            loaded = Trajectory.from_jsonl(path)
            assert len(loaded) == 1
            assert loaded[0].agent == "openhands"
            assert loaded[0].task.task_id == "test-001"
            assert len(loaded[0].steps) == 2

    def test_to_jsonl_append(self):
        traj = self._make_trajectory()

        with tempfile.TemporaryDirectory() as tmpdir:
            path = Path(tmpdir) / "trajectories.jsonl"
            traj.to_jsonl(path)
            traj.to_jsonl(path)

            loaded = Trajectory.from_jsonl(path)
            assert len(loaded) == 2

    def test_schema_json_example(self):
        schema_str = Trajectory.schema_json_example()
        schema = json.loads(schema_str)
        assert "properties" in schema
        assert "agent" in schema["properties"]

    def test_empty_steps(self):
        traj = Trajectory(
            task=TaskInfo(task_id="empty-001"),
            agent="test-agent",
            steps=[],
            outcome=Outcome(success=False, total_steps=0),
        )
        assert len(traj.steps) == 0
        assert traj.outcome.success is False

    def test_metadata(self):
        traj = self._make_trajectory()
        assert traj.metadata["run_id"] == "run-abc"
        assert traj.metadata["duration"] == 42.5
